import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import menu.*;

/**
* A PropertyPage is used to display lists of properties whose values
* can be obtained using the System.getProperty(keyString) or
* midlet.getAppProperty(keyString) methods.
* The list of properties that the PropertyPage displays is controlled
* by information provided to the constructor.
* <p>If the midlet parameter is null, then the requested properties
* are considered to be system properties and System.getProperty is
* used.  If midlet is not null, then midlet.getAppProperty is used.
* <p>A String[] can be provided containing the explicit names of the 
* properties.  In this case, the PropertyPage looks up the value for
* each property and then displays the name and the value.
*<p>Alternatively, the caller can provide a prefix String and a 
* String array containing the suffixes.  In this case, the PropertyPage
* builds the key strings itself using the format prefix[n]suffix.  Given
* a prefix, it adds the string "1", then adds each of the suffixes in
* turn, and uses the result as a key string.  Then n is incremented 
* and the process is repeated until for some value of n, none of the
* possible keys have a value.  Note that if you want hyphens before
* and after the "n", you need to put them in the prefix and suffix
* Strings, since they are NOT added automatically.
*/
public class PropertyPage extends Page {
	/**
	* Initialize a new PropertyPage object using the supplied parameters.
	* @param label short label for this page (as might be used in
	* a menu)
	* @param title longer label for this page (as might be used in
	* a screen title)
	* @param icon small icon for this page (as might be used in
	* a toolbar or menu
	* @param midlet the midlet for which these properties are stored.  If
	* null, then the properties are system properties.  If not, then they
	* are application properties.
	* @param prefix property name prefix String for generating a series
	* of property names.  If prefix is null, then no names are
	* generated, they are just used directly from the array given
	* in the suffix parameter.
	* @param suffix a String[] of property names or suffixes.
	*/
	public PropertyPage(String label,String title,Image icon,MIDlet midlet,String prefix,String[] suffix) {
		super(label,title,icon);
		if (suffix==null)
			throw new NullPointerException("Property names array must be supplied to "+this.getClass().getName());
		this.midlet = midlet;
		this.prefix = prefix;
		this.suffix = suffix;
	}
	/**
	* Initialize a new PropertyPage object using the supplied parameters
	* and default values where needed.
	* The title defaults to null (meaning that the label will be used
	* in its place).  
	* The icon defaults to null (meaning text only will be displayed).
	* @param label short label for this page (as might be used in
	* a menu)
	* @param midlet the midlet for which these properties are stored.  If
	* null, then the properties are system properties.  If not, then they
	* are application properties.
	* @param prefix property name prefix String for generating a series
	* of property names.  If prefix is null, then no names are
	* generated, they are just used directly from the array given
	* in the suffix parameter.
	* @param suffix a String[] of property name suffixes.
	*/
	public PropertyPage(String label,MIDlet midlet,String prefix,String[] names) {
		this(label,null,null,midlet,prefix,names);
	}
	/**
	* Initialize a new PropertyPage object using the supplied parameters
	* and default values where needed.
	* The title defaults to null (meaning that the label will be used
	* in its place).  
	* The icon defaults to null (meaning text only will be displayed).
	* The prefix defaults to null (meaning that no names are generated
	* automatically, they are just used as they are from the names parameter).
	* @param midlet the midlet for which these properties are stored.  If
	* null, then the properties are system properties.  If not, then they
	* are application properties.
	* @param label short label for this page (as might be used in
	* a menu)
	* @param names a String array containing the names of the
	* properties of interest.
	*/
	public PropertyPage(String label,MIDlet midlet,String[] names) {
		this(label,null,null,midlet,null,names);
	}
	/**
	* Process the user command and return an integer value telling
	* the PageMgr what to do next.  This method handles the details
	* of a command for a page, while the commandAction method in the
	* PageMgr handles the general responsibility of switching displays.
	* @param c a Command object identifying the command.
	* @param d the Displayable on which this event has occurred
	* @return a pageAction code, selected from the public fields defined
	* by PageMgr.
	*/
	public int doCommandAction(Command c,Displayable d) {
		int pageAction;
		if (d != template) {
			throw new IllegalArgumentException(
			"Unknown Displayable received in "+
			this.getClass().getName());
		}
		if (c == PageMgr.BACK_COMMAND) {
			pageAction = PageMgr.BACK;
		} else {
			throw new IllegalArgumentException(
			"Unknown Command received in "+
			this.getClass().getName());
		}
		return pageAction;
	}
	/**
	* Return the Displayable object for this PropertyPage, completely filled in
	* and ready to display.
	* @return the appropriate Displayable object for this Page
	*/
	public Displayable getDisplayable() {
		String pName;
		String pValue;
		int i;		// loop counter
		int n;		// loop limit
		n = template.size();
		for (i=n-1; i >= 0; i--) {
			template.delete(i);
		}
		int suffixCount = suffix.length;
		if (prefix == null) {
			for (i=0; i<suffixCount; i++) {
				pName = suffix[i];
				if (midlet == null) {
					pValue = System.getProperty(pName);
				} else {
					pValue = midlet.getAppProperty(pName);
				}
				if (pValue == null) pValue = "<not specified>";
				template.append(new StringItem(pName+": ",pValue));
			}
		} else {
			int group = 1;
			boolean found;
			do {
				found = false;
				for (i=0; i<suffixCount; i++) {
					pName = prefix+Integer.toString(group)+suffix[i];
					if (midlet == null) {
						pValue = System.getProperty(pName);
					} else {
						pValue = midlet.getAppProperty(pName);
					}
					if (pValue == null) continue;
					found = true;
					template.append(new StringItem(pName+": ",pValue));
				}
				group++;
			} while (found == true);
		}
				
		String t = (getTitle()==null) ? getLabel() : getTitle();
		template.setTitle(t);
		if (getPrevious() != null) {
			template.addCommand(PageMgr.BACK_COMMAND);
		} else {
			template.removeCommand(PageMgr.BACK_COMMAND);
		}
		return template;
	}

	//----------------------------------------------------------------------

	/** the template that is filled out to be our Displayable */
	static private PropertyPageScreen template = new PropertyPageScreen();
	
	/** 
	* the MIDlet for which these properties are defined or null if 
	* these are system properties.
	*/
	MIDlet midlet;
	
	/** the property name prefix String. */
	private String prefix;
	
	/** the property name suffixes (or complete names if prefix is null). */
	private String[] suffix;

	//--------------------------------------------------------------------
	
	/**
	* A PropertyPageScreen is the Displayable (the view) for a
	* PropertyPage (the model).
	*/
	static private class PropertyPageScreen extends Form {
		PropertyPageScreen() {
			super("Property Screen Template");
			addCommand(PageMgr.BACK_COMMAND);
		}
	}
}
